@@ -19,6 +19,7 @@
  */
 
 #include <dashboard_item.h>
+#include <dashboard_button.h>
 #include <scene.h>
 
 //#include <glib.h>
@@ -31,117 +32,21 @@ namespace
 
 const int width = 480;
 const int height = 160;
+const QSize extent = QSize(width, height);
+
 const int col_left_width = 120;
 const int col_right_width = 120;
 const int icon_size = 64;
-const int flash_duration = 200;
+
 const int fadeout_duration = 500;
 const QColor flash_color = QColor(Qt::red);
 
 }
 
-MDashboardButton::
-MDashboardButton(const QIcon &icon, QGraphicsItem *item, QObject *parent)
-: QObject(parent),
-  QGraphicsPixmapItem(icon.pixmap(icon_size, icon_size), item),
-  m_background(0),
-  m_stored_background_brush(QBrush(Qt::transparent)),
-  m_icon(icon),
-  m_active(true)
-{
-    setEnabled(true); // receive scene events
-}
-
-MDashboardButton::
-~MDashboardButton()
-{}
-
-void MDashboardButton::
-setupButtonBackground()
-{
-    const QRect ellipse_extent = QRect(0,0, icon_size, icon_size);
-    m_background = new QGraphicsEllipseItem(ellipse_extent, this);
-
-    QPen ellipse_border = QPen(Qt::white);
-    ellipse_border.setCosmetic(true);
-
-    m_background->setPen(ellipse_border);
-    m_background->setBrush(m_stored_background_brush);
-    m_background->show();
-    m_background->setEnabled(false);
-    m_background->setFlags(QGraphicsItem::ItemStacksBehindParent);
-
-    QGraphicsBlurEffect *blur = new QGraphicsBlurEffect(this);
-    blur->setBlurRadius(2);
-    m_background->setGraphicsEffect(blur);
-}
-
-void MDashboardButton::
-setBackgroundBrush(const QBrush &brush)
-{
-    if (!m_background)
-        return; // nothing to do!
-
-    m_background->setBrush(brush);
-}
-
-void MDashboardButton::
-storeBackgroundBrush()
-{
-    // Make sure to never store the brush's flash color, else triggering flash
-    // while a flash is already running becomes racy.
-    if (!m_background || flash_color ==  m_background->brush().color())
-        return; // nothing to do!
-
-    m_stored_background_brush = m_background->brush();
-}
-
-void MDashboardButton::
-restoreBackgroundBrush()
-{
-    setBackgroundBrush(m_stored_background_brush);
-}
-
-void MDashboardButton::
-flash()
-{
-    storeBackgroundBrush();
-    setBackgroundBrush(QBrush(flash_color));
-    QTimer::singleShot(flash_duration, this, SLOT(restoreBackgroundBrush()));
-}
-
-
-void MDashboardButton::
-mousePressEvent(QGraphicsSceneMouseEvent *event)
-{
-    event->accept(); // stop event propagation
-
-    if (m_background)
-    {
-        const QBrush flash = QBrush(Qt::red);
-        m_background->setBrush(flash);
-    }
-
-    Q_EMIT pressed();
-}
-
-void MDashboardButton::
-mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
-{
-    event->accept(); // stop event propagation
-
-    if (m_background)
-    {
-        const QBrush trans = QBrush(Qt::transparent);
-        m_background->setBrush(trans);
-    }
-
-    Q_EMIT released();
-}
-
 MDashboardItem::
-MDashboardItem(QGraphicsItem *parent)
-: QGraphicsObject(parent),
+MDashboardItem(QGraphicsAnchorLayout *layout, QGraphicsItem *parent, Qt::WindowFlags flags)
+: QGraphicsWidget(parent, flags),
+  m_layout(layout),
   m_confirm(0),
   m_requests(0),
   m_takeback(0),
@@ -166,7 +71,7 @@ MDashboardItem::
 void MDashboardItem::
 setupUi()
 {
-    // TODO: use anchor layouts, this is a fantastic playground for them!
+    setPreferredSize(extent);
 
     // Maemo5-specific icons, handle press states by replacing pixmap, or have two pixmap per item?
     const int centered_width  = (width  - icon_size) * 0.5;
@@ -177,16 +82,23 @@ setupUi()
     m_requests_dialog->setPalette(palette);
     m_requests_dialog->setZValue(1); // make sure it is drawn on top of its siblings, the buttons.
     m_requests_dialog->setEnabled(false);
+    m_layout->addAnchor(this,              Qt::AnchorBottom,
+                        m_requests_dialog, Qt::AnchorBottom);
+
     m_requests_dialog->hide();
 
     m_game_dialog->setPalette(palette);
     m_game_dialog->setZValue(1);
     m_game_dialog->setEnabled(false);
+    m_layout->addAnchor(this,          Qt::AnchorBottom,
+                        m_game_dialog, Qt::AnchorBottom);
     m_game_dialog->hide();
 
     m_offers_dialog->setPalette(palette);
     m_offers_dialog->setZValue(1);
     m_offers_dialog->setEnabled(false);
+    m_layout->addAnchor(this,            Qt::AnchorBottom,
+                        m_offers_dialog, Qt::AnchorBottom);
     m_offers_dialog->hide();
 
     QFont font;
@@ -224,27 +136,40 @@ setupUi()
     m_last_moves->setTextWidth(col_right_width);
     m_last_moves->setPos(QPoint(width - col_right_width, 0)); // below the board, right-aligned
 
-    m_confirm = createButtonWithBackground(QPoint(centered_width, centered_height),
-                                           QIcon("/usr/share/themes/alpha/mediaplayer/Play.png"));
+    m_confirm = new MDashboardButton(QPoint(centered_width - 16, centered_height - 16),
+                                     QPixmap(":/icons/96x96/play.png"),
+                                     QPixmap(":/icons/96x96/play-inactive.png"),
+                                     QPixmap(":/icons/96x96/play-pressed.png"),
+                                     QPixmap(":/icons/96x96/play-flash.png"),
+                                     this);
+    m_confirm->setActive(false);
 
     // right-aligned
-    m_requests = createButtonWithBackground(QPoint(width - icon_size - col_right_width, centered_height),
-                                           QIcon("/usr/share/themes/alpha/mediaplayer/Stop.png"));
+    m_requests = new MDashboardButton(QPoint(width - icon_size - col_right_width, centered_height),
+                                      QPixmap(":/icons/64x64/stop.png"),
+                                      QPixmap(":/icons/64x64/stop-inactive.png"),
+                                      QPixmap(":/icons/64x64/stop-pressed.png"),
+                                      QPixmap(":/icons/64x64/stop-flash.png"),
+                                      this);
+    m_requests->setActive(false);
 
     // left-aligned
-    m_takeback = createButtonWithBackground(QPoint(col_left_width, centered_height),
-                                            QIcon("/usr/share/themes/alpha/mediaplayer/Back.png"));
+    m_takeback = new MDashboardButton(QPoint(col_left_width, centered_height),
+                                      QPixmap(":/icons/64x64/rewind.png"),
+                                      QPixmap(":/icons/64x64/rewind-inactive.png"),
+                                      QPixmap(":/icons/64x64/rewind-pressed.png"),
+                                      QPixmap(":/icons/64x64/rewind-flash.png"),
+                                      this);
+    m_takeback->setActive(false); // TODO: not implemented yet
 
     QPixmap *avatar = getContactsAvatar(QString("qgil"));
     QPixmap empty;
     if (avatar)
-        m_avatar = new MDashboardButton(*avatar, this, 0);
+        m_avatar = new MDashboardButton(QPoint(10, centered_height),
+                                        *avatar, *avatar, *avatar, *avatar, this, 0);
     else
-       m_avatar = new MDashboardButton(empty, this, 0);
-
-    m_avatar->setPos(QPoint(10, centered_height)); // left-aligned
-    m_avatar->show();
-    m_avatar->setEnabled(true);
+       m_avatar = new MDashboardButton(QPoint(10, centered_height),
+                                       empty, empty, empty, empty, this, 0);
 
     // now that the buttons exist we can connect their signals (or rather, forward them)
     connect(m_confirm, SIGNAL(pressed()), this, SIGNAL(confirmButtonPressed()));
@@ -264,26 +189,13 @@ resetUi()
     m_requests_dialog->hide();
     m_offers_dialog->hide();
 
-    m_confirm->restoreBackgroundBrush();
-    m_requests->restoreBackgroundBrush();
-    m_takeback->restoreBackgroundBrush();
+    m_confirm->restore();
+    m_requests->restore();
+    m_takeback->restore();
 }
 
-MDashboardButton * MDashboardItem::
-createButtonWithBackground(const QPoint &origin, const QIcon &icon)
-{
-    MDashboardButton *button = new MDashboardButton(icon, this, 0);
-    button->setPos(origin);
-    button->show();
-    button->setEnabled(true);
-    button->setTransformationMode(Qt::SmoothTransformation);
-    button->setupButtonBackground();
-
-    return button;
-}
 
 // MDashboardItem private:
-
 QPixmap * MDashboardItem::
 getContactsAvatar(const QString &nick)
 {
@@ -308,23 +220,31 @@ showConfirmationDialog(const QString &title, QPushButton *button)
 {
     static_cast<MScene *>(scene())->setModalItem(m_offers_dialog);
 
-    QWidget *dialog = new QWidget;
-    dialog->resize(width, height);
-
-    QVBoxLayout *vbox = new QVBoxLayout;
-    dialog->setLayout(vbox);
-
-    QLabel *label = new QLabel(title);
-    label->setAlignment(Qt::AlignCenter);
-    vbox->addWidget(label);
-
-    if (button)
+    if (!m_offers_dialog->widget())
     {
-        vbox->addWidget(button);
-        connect(button, SIGNAL(pressed()), dialog, SLOT(hide()));
+        QWidget *dialog = new QWidget;
+        dialog->resize(width, height);
+
+        QVBoxLayout *vbox = new QVBoxLayout;
+        dialog->setLayout(vbox);
+
+        QLabel *label = new QLabel(title);
+        label->setAlignment(Qt::AlignCenter);
+        vbox->addWidget(label);
+
+        if (button)
+        {
+            vbox->addWidget(button);
+            connect(button, SIGNAL(pressed()), dialog, SLOT(hide()));
+        }
+
+        m_offers_dialog->setWidget(dialog);
     }
 
-    m_offers_dialog->setWidget(dialog);
+    // Need to manually transform proxy widgets, since the parent is a QGraphicsWidget
+    // (managed by a layout), and this is wonderfully borken.
+    m_offers_dialog->setTransformOriginPoint(m_offers_dialog->boundingRect().center());
+    m_offers_dialog->setRotation(rotation());
     m_offers_dialog->show();
     m_offers_dialog->setEnabled(true);
 }
@@ -345,8 +265,12 @@ showEndGameUi()
     QWidget *dialog = new QWidget;
     dialog->resize(width, height - 60);
     dialog_proxy->setWidget(dialog);
+
+    // Need to manually transform proxy widgets, since the parent is a QGraphicsWidget
+    // (managed by a layout), and this is wonderfully borken.
+    dialog_proxy->setTransformOriginPoint(dialog_proxy->boundingRect().center());
+    dialog_proxy->setRotation(rotation());
     dialog_proxy->show();
-    dialog_proxy->setPos(QPoint(0, 60));
 
     QVBoxLayout *vbox = new QVBoxLayout;
     dialog->setLayout(vbox);
@@ -372,9 +296,7 @@ enableConfirmButton()
 {
     Q_CHECK_PTR(m_confirm);
 
-    m_confirm->setEnabled(true);
-    m_confirm->setBackgroundBrush(QBrush(Qt::green));
-    m_confirm->storeBackgroundBrush();
+    m_confirm->setActive(true);
 }
 
 void MDashboardItem::
@@ -382,9 +304,7 @@ disableConfirmButton()
 {
     Q_CHECK_PTR(m_confirm);
 
-    m_confirm->setEnabled(false);
-    m_confirm->setBackgroundBrush(QBrush(Qt::transparent));
-    m_confirm->storeBackgroundBrush();
+    m_confirm->setActive(false);
 }
 
 void MDashboardItem::
@@ -392,7 +312,7 @@ enableRequestsButton()
 {
     Q_CHECK_PTR(m_requests);
 
-    m_requests->setEnabled(true);
+    m_requests->setActive(true);
 }
 
 void MDashboardItem::
@@ -400,8 +320,7 @@ disableRequestsButton()
 {
     Q_CHECK_PTR(m_requests);
 
-    m_requests->setEnabled(false);
-    m_requests->setBackgroundBrush(Qt::transparent);
+    m_requests->setActive(false);
 }
 
 void MDashboardItem::
@@ -451,8 +370,11 @@ showRequestsMenu()
         dialog->show();
     }
 
+    // Need to manually transform proxy widgets, since the parent is a QGraphicsWidget
+    // (managed by a layout), and this is wonderfully borken.
+    m_requests_dialog->setTransformOriginPoint(m_requests_dialog->boundingRect().center());
+    m_requests_dialog->setRotation(rotation());
     m_requests_dialog->show();
-    m_requests_dialog->setPos(QPoint(0, -60));
     m_requests_dialog->setEnabled(true);
 }
 
@@ -489,6 +411,10 @@ showGameMenu()
         dialog->show();
     }
 
+    // Need to manually transform proxy widgets, since the parent is a QGraphicsWidget
+    // (managed by a layout), and this is wonderfully borken.
+    m_game_dialog->setTransformOriginPoint(m_game_dialog->boundingRect().center());
+    m_game_dialog->setRotation(rotation());
     m_game_dialog->show();
     m_game_dialog->setEnabled(true);
 }
